/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Functions for interaction with the Asterisk database
 * 
 * Copyright (C) 2005, Russell Bryant <russelb@clemson.edu> 
 *	
 * CountryCode: Calculates what portion of a digit string is the country code
 * Copyright (C) 2003, Vikram Rangnekar <vicram@gmail.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <regex.h>

#include "asterisk/pbx.h"
#include "asterisk/logger.h"

static int findCCodeInPatt(char *buff, int len, char *pattern)
{
	int numdigs=2; /* Set default length to 2 digits */
	int d1=0;
	int d2=0;

	d1 = pattern[0]-'0';
	d2 = pattern[1]-'0';

	if(len < 5) {
		ast_log(LOG_WARNING, "Buffer is too small to hold return value, len: %d\n", len);
		return 1;
	}
	buff[0]='\0';
			
	switch (d1) {
	case 1:
	case 7:
		numdigs=1;
		break;
	case 2:
		if(d2!=0 && d2!=7) 
			numdigs=3;
		break;
	case 3:
		if(d2==5 || d2==7 || d2==8) 
			numdigs=3;
		break;
	case 4:
		if(d2==2)
			numdigs=3;
		break;
	case 5:
		if(d2==0 || d2==9)
			numdigs=3;
		break;
	case 6:
		if(d2 >= 7)
			numdigs=3;
		break;
	case 8:
		if(d2==0 || d2==3 || d2==5 || d2>=7) 
			numdigs=3;
		break;
	case 9:
		if(d2==6 || d2==7 || d2==9) 
			numdigs=3;
		break;
	default:
		numdigs=0;
	} /* end switch (d1) */

	if (d2<0 || d2>9) {
		/* No good if second digit is not a digit */
		numdigs=0;
	}
	else {
		if (strlen(pattern) < numdigs) {
			/* No good if pattern not long enough */
			numdigs=0;
		}
		else {
			/* Copy country code portion of pattern into buff */
			strncpy(buff,pattern,numdigs);
			buff[numdigs]='\0';
	 	}
	}
	return(numdigs);
}

static char *function_getcc(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len)
{
	char *args;

	if (!data || ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "CountryCode requires an argument, CountryCode(<extension>)\n");
		return buf;
	}

	if (findCCodeInPatt(buf, len, data) == 0) 
		ast_log(LOG_DEBUG,  "Bad number pattern '%s'. Make sure the IAD is stripped.\n", data);

	
	return buf;
}

#ifndef BUILTIN_FUNC
static
#endif
struct ast_custom_function countrycode_function = {
	.name = "COUNTRYCODE",
	.synopsis = "Calculates what portion of a digit string is the country code",
	.syntax = "COUNTRYCODE(<extension>)",
	.desc = 
	"  COUNTRYCODE(varname=${EXTEN}): Set a variable to the country code portion of the extension\n"
	"Calculates what portion of a digit string is the country code using\n"
	"known country code length patterns used throughout the world.  Can\n"
	"analyze number without a database hit\n\n"
	"The input 'pattern' to the routine MUST NOT contain the leading\n"
	"International Access digits for the country you are in.  (EXAMPLE:  Do not\n"
	"have the leading '011' if in the US, or '00/01' in other countries)\n\n"
	"watch how the routine, when given something like 52583839337737 will find\n"
	"and spit back 52 as the country code\n",
	.read = function_getcc,
};
